@@ -2,7 +2,8 @@
 # SPDX-License-Identifier: GPL-3.0-only
 from gi.repository import Gtk, GObject, Gdk
 
-from pyatem.command import KeyPropertiesDveCommand, DkeyMaskCommand, KeyPropertiesAdvancedChromaColorpickerCommand
+from pyatem.command import KeyPropertiesDveCommand, DkeyMaskCommand, KeyPropertiesAdvancedChromaColorpickerCommand, \
+    SupersourceBoxPropertiesCommand
 
 
 class Region:
@@ -152,11 +153,44 @@ class ColorPickerRegion(Region):
         return [cmd]
 
 
+class SuperSourceBoxRegion(Region):
+    def get_label(self):
+        return _("SuperSource Box {}").format(self.rid[1] + 1)
+
+    def _update_region(self, index, pos_x=None, pos_y=None, size_x=None, size_y=None):
+        if pos_x is not None:
+            x = int((pos_x * 2 - 1) * 1600)
+        if pos_y is not None:
+            y = int((pos_y * 2 - 1) * 900)
+
+        size = None
+        if size_x is not None and size_y is not None:
+            size = int(size_x * 1000)
+        cmd = SupersourceBoxPropertiesCommand(index=0, box=self.rid[1], x=x, y=y,
+                                              size=size)
+        return [cmd]
+
+    def _update_mask(self, index, left=None, right=None, top=None, bottom=None):
+        ml, mr, mt, mb = None, None, None, None
+        if left is not None:
+            ml = int(left * 32000)
+        if right is not None:
+            mr = int(right * 32000)
+        if top is not None:
+            mt = int(top * 18000)
+        if bottom is not None:
+            mb = int(bottom * 18000)
+
+        cmd = SupersourceBoxPropertiesCommand(index=0, box=self.rid[1], top=mt, bottom=mb, left=ml, right=mr)
+        return [cmd]
+
+
 class LayoutView(Gtk.Frame):
     __gtype_name__ = 'LayoutView'
     LAYER_USK = 1
     LAYER_DSK = 2
     LAYER_ACK = 3
+    LAYER_SSB = 4
 
     def __init__(self, index, connection):
         super(Gtk.Frame, self).__init__()
@@ -225,10 +259,36 @@ class LayoutView(Gtk.Frame):
                 region = DownstreamKeyRegion(rid, self)
             elif type == LayoutView.LAYER_ACK:
                 region = ColorPickerRegion(rid, self)
+            elif type == LayoutView.LAYER_SSB:
+                region = SuperSourceBoxRegion(rid, self)
             self.regions[rid] = region
         return self.regions[rid]
 
+    def on_context(self):
+        menu = Gtk.Menu()
+        for rid in self.regions:
+            region = self.regions[rid]
+            mi = Gtk.CheckMenuItem(region.get_label())
+            mi.set_active(region.visible)
+            menu.append(mi)
+            mi.show()
+            mi.rid = rid
+            mi.connect('toggled', self.on_context_toggled)
+        menu.popup(None, None, None, None, 0, Gtk.get_current_event_time())
+
+    def on_context_toggled(self, widget, *args):
+        self.regions[widget.rid].visible = widget.get_active()
+        self.da.queue_draw()
+
     def on_mouse_down(self, widget, event):
+
+        if event.button == 3:
+            self.on_context()
+            return
+
+        if event.button != 1:
+            return
+
         # grr cairo coordinates
         event.y = self.area_height - event.y - self.area_top
         event.x = event.x - self.area_left
@@ -238,6 +298,8 @@ class LayoutView(Gtk.Frame):
             hits = []
             for rid in self.regions:
                 region = self.regions[rid]
+                if not region.visible:
+                    continue
                 x, y, w, h = region.get_region_cairo(self.area_width, self.area_height)
 
                 if x < event.x < (x + w):
