@@ -2,6 +2,7 @@
 # SPDX-License-Identifier: GPL-3.0-only
 import gettext
 import logging
+import re
 from datetime import datetime, timedelta
 
 from gtk_switcher.layout import LayoutView
@@ -11,7 +12,7 @@ from pyatem.command import CutCommand, AutoCommand, FadeToBlackCommand, Transiti
     DkeyAutoCommand, DkeyTieCommand, \
     DkeyOnairCommand, ProgramInputCommand, PreviewInputCommand, KeyOnAirCommand, KeyFillCommand, \
     FadeToBlackConfigCommand, RecorderStatusCommand, AuxSourceCommand, StreamingServiceSetCommand, \
-    RecordingSettingsSetCommand, StreamingStatusSetCommand, MediaplayerSelectCommand
+    RecordingSettingsSetCommand, StreamingStatusSetCommand, MediaplayerSelectCommand, StreamingAudioBitrateCommand
 from pyatem.field import TransitionSettingsField, InputPropertiesField, TopologyField
 import gtk_switcher.stream_data
 
@@ -47,11 +48,13 @@ class SwitcherPage:
         self.wipe_reverse = builder.get_object('wipe_reverse')
         self.wipe_flipflop = builder.get_object('wipe_flipflop')
 
+        self.dve_reverse = builder.get_object('dve_reverse')
+        self.dve_flipflop = builder.get_object('dve_flipflop')
+
         self.ftb_afv = builder.get_object('ftb_afv')
 
         self.keyer_stack = builder.get_object('keyer_stack')
 
-        self.usk_count = 0
         self.usks = {}
         self.dsks = {}
         self.has_models = []
@@ -102,6 +105,11 @@ class SwitcherPage:
         self.stream_live_active = False
         self.stream_live_start_time = None
 
+        self.flap = builder.get_object('flap')
+        self.flaptoggle = builder.get_object('flaptoggle')
+
+        self.flap.bind_property("reveal-flap", self.flaptoggle, "active", GObject.BindingFlags.BIDIRECTIONAL)
+
         self.switcher_mediaplayers = builder.get_object('switcher_mediaplayers')
         self.mediaplayer_dropdowns = {}
 
@@ -141,6 +149,24 @@ class SwitcherPage:
             button.pattern = style
             button.connect('pressed', self.on_wipe_pattern_clicked)
 
+        directions = ['tl', 't', 'tr', 'l', 'r', 'bl', 'b', 'br']
+        self.dve_style_push = []
+        self.dve_style_squeeze = []
+        for stylecode, direction in enumerate(directions):
+            btn = builder.get_object(f'dt_push_{direction}')
+            btn.style = 'push'
+            btn.direction = direction
+            btn.styleindex = stylecode + 24
+            btn.connect('pressed', self.on_dve_transition_style_clicked)
+            self.dve_style_push.append(btn)
+
+            btn = builder.get_object(f'dt_squeeze_{direction}')
+            btn.style = 'squeeze'
+            btn.direction = direction
+            btn.styleindex = stylecode + 16
+            btn.connect('pressed', self.on_dve_transition_style_clicked)
+            self.dve_style_squeeze.append(btn)
+
         self.color1 = builder.get_object('color1')
         self.color2 = builder.get_object('color2')
 
@@ -241,6 +267,27 @@ class SwitcherPage:
         cmd = WipeSettingsCommand(index=0, pattern=widget.pattern)
         self.connection.mixer.send_commands([cmd])
 
+    def on_dve_transition_style_clicked(self, widget):
+        if self.model_changing:
+            return
+
+        cmd = DveSettingsCommand(index=0, style=widget.styleindex)
+        self.connection.mixer.send_commands([cmd])
+
+    def on_dve_reverse_clicked(self, widget):
+        if self.model_changing:
+            return
+        state = widget.get_style_context().has_class('active')
+        cmd = DveSettingsCommand(index=0, reverse=not state)
+        self.connection.mixer.send_commands([cmd])
+
+    def on_dve_flipflop_clicked(self, widget):
+        if self.model_changing:
+            return
+        state = widget.get_style_context().has_class('active')
+        cmd = DveSettingsCommand(index=0, flipflop=not state)
+        self.connection.mixer.send_commands([cmd])
+
     def on_tbar_position_changed(self, widget, index, position):
         cmd = TransitionPositionCommand(index=index, position=position)
         self.connection.mixer.send_commands([cmd])
@@ -461,6 +508,13 @@ class SwitcherPage:
             self.log_sw.warning("Got transition dve change for non-existing M/E {}".format(data.index + 1))
             return
 
+        for button in self.dve_style_push + self.dve_style_squeeze:
+            self.set_class(button, 'stylebtn', True)
+            self.set_class(button, 'active', data.style == button.styleindex)
+
+        self.set_class(self.dve_reverse, 'active', data.reverse)
+        self.set_class(self.dve_flipflop, 'active', data.flipflop)
+
         label = self.frames_to_time(data.rate)
         self.dve_rate.set_text(label)
         self.me[data.index].set_auto_rate('dve', data.rate)
@@ -486,6 +540,35 @@ class SwitcherPage:
         region = self.layout[0].get(LayoutView.LAYER_DSK, data.index)
         region.set_tally(data.on_air)
 
+    def make_preset_expander(self, label, context, widget):
+        overlay = Gtk.Overlay()
+        expander = Gtk.Expander()
+        expander.get_style_context().add_class('bmdgroup')
+        frame_label = Gtk.Label(label)
+        frame_label.get_style_context().add_class("heading")
+        expander.set_label_widget(frame_label)
+        expander.set_expanded(True)
+        expander.add(widget)
+        overlay.add(expander)
+
+        button = Gtk.MenuButton()
+        button.set_valign(Gtk.Align.START)
+        button.set_halign(Gtk.Align.END)
+        self.set_class(button, 'flat', True)
+        self.set_class(button, 'preset', True)
+        self.apply_css(button, self.provider)
+        overlay.add_overlay(button)
+
+        hamburger = Gtk.Image.new_from_icon_name('open-menu-symbolic', Gtk.IconSize.BUTTON)
+        button.set_image(hamburger)
+        button.set_name(context)
+
+        context = context.split(':')
+        button.set_menu_model(self.preset_models[context[0]])
+
+        button.connect("clicked", self.on_presets_clicked)
+        return overlay
+
     def on_topology_change(self, data: TopologyField):
         # Create the M/E units
         for i in range(0, data.me_units - len(self.me)):
@@ -500,14 +583,10 @@ class SwitcherPage:
             self.downstream_keyers.remove(widget)
 
         for i in range(0, data.downstream_keyers):
-            exp = Gtk.Expander()
-            exp.get_style_context().add_class('bmdgroup')
             label = _("Downstream keyer {}").format(i + 1)
-            frame_label = Gtk.Label(label)
-            frame_label.get_style_context().add_class("heading")
-            exp.set_label_widget(frame_label)
-            exp.set_expanded(True)
             dsk = DownstreamKeyer(index=i, connection=self.connection)
+            exp = self.make_preset_expander(label, f"dsk:{dsk.index}", dsk)
+
             self.dsks[dsk.index] = dsk
             self.has_models.append(dsk)
             exp.add(dsk)
@@ -611,23 +690,29 @@ class SwitcherPage:
             self.log_sw.warning("Got _MeC for non-existing M/E {}".format(data.index + 1))
             return
 
+        if data.index not in self.usks:
+            self.usks[data.index] = {}
+
         self.me[data.index].set_config(data)
 
         from gtk_switcher.upstreamkey import UpstreamKeyer
 
-        if data.keyers > self.usk_count:
-            add = data.keyers - self.usk_count
+        usk_count = len(self.usks[data.index])
+        if data.keyers > usk_count:
+            add = data.keyers - usk_count
 
             for i in range(0, add):
-                self.usk_count += 1
                 exp = Gtk.Expander()
                 exp.get_style_context().add_class('bmdgroup')
-                frame_label = Gtk.Label(_("Upstream keyer {}").format(self.usk_count))
+                label_text = _("Upstream keyer {}").format(len(self.usks[data.index]) + 1)
+                if data.index != 0:
+                    label_text += ' [M/E {}]'.format(data.index + 1)
+                frame_label = Gtk.Label(label_text)
                 frame_label.get_style_context().add_class("heading")
                 exp.set_label_widget(frame_label)
                 exp.set_expanded(True)
-                usk = UpstreamKeyer(index=0, keyer=self.usk_count - 1, connection=self.connection)
-                self.usks[usk.index] = usk
+                usk = UpstreamKeyer(index=data.index, keyer=len(self.usks[data.index]), connection=self.connection)
+                self.usks[data.index][usk.keyer] = usk
                 self.has_models.append(usk)
                 exp.add(usk)
                 self.apply_css(usk, self.provider)
@@ -687,25 +772,38 @@ class SwitcherPage:
         self.connection.mixer.send_commands([cmd])
 
     def on_key_properties_base_change(self, data):
-        if data.keyer not in self.usks:
+        if data.index not in self.usks:
+            self.log_sw.warning(
+                "Got key-properties-base for non-existant M/E {}".format(data.index + 1))
+            return
+        if data.keyer not in self.usks[data.index]:
             self.log_sw.warning(
                 "Got key-properties-base for non-existant keyer {} M/E {}".format(data.keyer, data.index + 1))
             return
-        self.usks[data.index].on_key_properties_base_change(data)
+        self.usks[data.index][data.keyer].on_key_properties_base_change(data)
 
     def on_key_properties_luma_change(self, data):
-        if data.keyer not in self.usks:
+        if data.index not in self.usks:
+            self.log_sw.warning(
+                "Got key-properties-luma for non-existant M/E {}".format(data.index + 1))
+            return
+        if data.keyer not in self.usks[data.index]:
             self.log_sw.warning(
                 "Got key-properties-luma for non-existant keyer {} M/E {}".format(data.keyer, data.index + 1))
             return
-        self.usks[data.index].on_key_properties_luma_change(data)
+        self.usks[data.index][data.keyer].on_key_properties_luma_change(data)
 
     def on_key_properties_dve_change(self, data):
-        if data.keyer not in self.usks:
+        if data.index not in self.usks:
+            self.log_sw.warning(
+                "Got key-properties-dve for non-existant M/E {}".format(data.index + 1))
+            return
+        if data.keyer not in self.usks[data.index]:
             self.log_sw.warning(
                 "Got key-properties-dve for non-existant keyer {} M/E {}".format(data.keyer, data.index + 1))
             return
-        self.usks[data.keyer].on_key_properties_dve_change(data)
+
+        self.usks[data.index][data.keyer].on_key_properties_dve_change(data)
         width = 16.0 * data.size_x / 1000
         height = 9.0 * data.size_y / 1000
 
@@ -714,16 +812,27 @@ class SwitcherPage:
         region.set_mask(data.mask_top, data.mask_bottom, data.mask_left, data.mask_right)
 
     def on_key_properties_advanced_chroma_change(self, data):
-        if data.keyer not in self.usks:
-            self.log_sw.warning("Got KACk for non-existant keyer {} M/E {}".format(data.keyer, data.index + 1))
+        if data.index not in self.usks:
+            self.log_sw.warning(
+                "Got KACk for non-existant M/E {}".format(data.index + 1))
             return
-        self.usks[data.keyer].on_advanced_chroma_change(data)
+        if data.keyer not in self.usks[data.index]:
+            self.log_sw.warning(
+                "Got KACk for non-existant keyer {} M/E {}".format(data.keyer, data.index + 1))
+            return
+        self.usks[data.index][data.keyer].on_advanced_chroma_change(data)
 
     def on_key_properties_advanced_chroma_colorpicker_change(self, data):
-        if data.keyer not in self.usks:
-            self.log_sw.warning("Got KACC for non-existant keyer {} M/E {}".format(data.keyer, data.index + 1))
+        if data.index not in self.usks:
+            self.log_sw.warning(
+                "Got KACC for non-existant M/E {}".format(data.index + 1))
             return
-        self.usks[data.keyer].on_chroma_picker_change(data)
+        if data.keyer not in self.usks[data.index]:
+            self.log_sw.warning(
+                "Got KACC for non-existant keyer {} M/E {}".format(data.keyer, data.index + 1))
+            return
+
+        self.usks[data.index][data.keyer].on_chroma_picker_change(data)
 
         size = data.size / 1000
 
@@ -887,6 +996,13 @@ class SwitcherPage:
         mp = []
         black = None
         bars = None
+        extra1 = []
+        extra2 = []
+
+        extra1_me1 = []
+        extra2_me1 = []
+        extra1_me2 = []
+        extra2_me2 = []
 
         # Clear the combobox models
         self.model_changing = True
@@ -907,6 +1023,13 @@ class SwitcherPage:
                 black = i
             if i.port_type == InputPropertiesField.PORT_BARS:
                 bars = i
+            if i.port_type == InputPropertiesField.PORT_SUPERSOURCE:
+                extra2.append(i)
+            if i.port_type == InputPropertiesField.PORT_ME_OUTPUT:
+                if i.available_me1:
+                    extra1_me1.append(i)
+                if i.available_me2:
+                    extra1_me2.append(i)
 
             if i.available_me1:
                 self.model_me1_fill.append([str(i.index), i.name])
@@ -966,13 +1089,20 @@ class SwitcherPage:
             row1_ext = external[0:num]
             row2_ext = external[num:] + [None] * ((2 * num) - len(external))
 
-        row1 = row1_ext + [None, black, None] + colors
-        row2 = row2_ext + [None, bars, None] + mp
+        row1 = row1_ext + [None, black, None] + colors + extra1
+        row2 = row2_ext + [None, bars, None] + mp + extra2
 
         buttons = [row1, row2]
 
         for me in self.me:
-            me.set_inputs(buttons)
+            me_btns = [buttons[0][:], buttons[1][:]]
+            if me.index == 0:
+                me_btns[0] += extra1_me1
+                me_btns[1] += extra2_me1
+            elif me.index == 1:
+                me_btns[0] += extra1_me2
+                me_btns[1] += extra2_me2
+            me.set_inputs(me_btns)
             self.apply_css(me, self.provider)
 
         self.model_changing = False
@@ -1088,11 +1218,21 @@ class SwitcherPage:
         self.macro_edit = True
         self.connection.mixer.download(0xffff, widget.index)
 
+    def bps_to_human(self, bps):
+        if bps < 1000:
+            return str(bps) + 'b'
+        elif bps < 1000 * 1000:
+            val = bps / 1000
+            return f'{val:g}k'
+        else:
+            val = bps / 1000 / 1000
+            return f'{val:g}M'
+
     def on_streaming_service_change(self, data):
         self.expander_livestream.show()
         self.expander_encoder.show()
-        self.video_rate_min.set_text(str(int(data.min / 1000)))
-        self.video_rate_max.set_text(str(int(data.max / 1000)))
+        self.video_rate_min.set_text(self.bps_to_human(data.min))
+        self.video_rate_max.set_text(self.bps_to_human(data.max))
 
         self.stream_live_platform.set_text(data.name)
         self.stream_live_server.set_text(data.url)
@@ -1100,8 +1240,8 @@ class SwitcherPage:
 
     def on_streaming_audio_bitrate_change(self, data):
         self.expander_encoder.show()
-        self.audio_rate_min.set_text(str(int(data.min / 1000)))
-        self.audio_rate_max.set_text(str(int(data.max / 1000)))
+        self.audio_rate_min.set_text(self.bps_to_human(data.min))
+        self.audio_rate_max.set_text(self.bps_to_human(data.max))
 
     def on_streaming_status_change(self, data):
         starting = data.status == 2
@@ -1194,3 +1334,54 @@ class SwitcherPage:
     def on_stream_recorder_filename_activate(self, widget, *args):
         cmd = RecordingSettingsSetCommand(filename=widget.get_text())
         self.connection.mixer.send_commands([cmd])
+
+    def human_to_bps(self, human):
+        human = human.lower()
+        human = human.replace('ps', '')
+
+        if human.isnumeric():
+            num = human
+            unit = ''
+        else:
+            if not ' ' in human:
+                human = re.sub(r'([a-z]+)', r' \1', human)
+            num, unit = [string.strip() for string in human.split()]
+
+        mult = 1
+        if unit.startswith('k'):
+            mult = 1000
+        elif unit.startswith('m'):
+            mult = 1000 * 1000
+        return int(float(num) * mult)
+
+    def on_audio_bitrate_activate(self, widget, *args):
+        rate_min = self.human_to_bps(self.audio_rate_min.get_text())
+        rate_max = self.human_to_bps(self.audio_rate_max.get_text())
+        if rate_max < rate_min:
+            rate_max = rate_min
+        cmd = StreamingAudioBitrateCommand(rate_min, rate_max)
+        self.connection.mixer.send_commands([cmd])
+
+    def on_video_bitrate_activate(self, widget, *args):
+        rate_min = self.human_to_bps(self.video_rate_min.get_text())
+        rate_max = self.human_to_bps(self.video_rate_max.get_text())
+        if rate_max < rate_min:
+            rate_max = rate_min
+        cmd = StreamingServiceSetCommand(bitrate_min=rate_min, bitrate_max=rate_max)
+        self.connection.mixer.send_commands([cmd])
+
+    def on_fold_folded(self, widget, *args):
+        self.flaptoggle.set_visible(self.flap.get_folded())
+
+    def on_presets_clicked(self, widget, *args):
+        self.preset_context = widget.get_name()
+
+    def on_supersource_box_change(self, data):
+        x = data.x / 100
+        y = data.y / 100
+        width = data.size / 110 * 16 / 9
+        height = data.size / 110
+
+        region = self.layout[0].get(LayoutView.LAYER_SSB, data.box)
+        region.set_region(x, y, width, height)
+        region.set_mask(data.mask_top, data.mask_bottom, data.mask_left, data.mask_right)
