@@ -1,6 +1,7 @@
 # Copyright 2021 - 2022, Martijn Braam and the OpenAtem contributors
 # SPDX-License-Identifier: LGPL-3.0-only
 import colorsys
+import datetime
 import struct
 
 
@@ -1228,7 +1229,11 @@ class FairlightMasterPropertiesCommand(Command):
 
     def __init__(self, eq_gain=None, dynamics_gain=None, volume=None, afv=None, eq_enable=None):
         """
-        :param index: 0-indexed M/E number to control the preview bus of
+        :param eq_gain: Make-up gain for the master EQ dialog
+        :param eq_enable: Enable the EQ on the master bus
+        :param dynamics_gain: Make-up gain for the master Dynamics dialog
+        :param volume: Master volume fader value
+        :param afv: Make the master volume follow the fade-to-black function
         """
 
         self.eq_gain = eq_gain
@@ -1264,6 +1269,11 @@ class FairlightStripPropertiesCommand(Command):
     """
     Implementation of the `CFSP` command. This sets the settings of a channel strip in fairlight audio.
 
+    The source index is the same as the video source index from the InputProperties fields.
+
+    For normal channel strips the channel index should be -1. If the channel is split (two mono channels instead of a
+    stereo input) the channel numbers are 0 and 1.
+
     ====== ==== ====== ===========
     Offset Size Type   Description
     ====== ==== ====== ===========
@@ -1300,7 +1310,16 @@ class FairlightStripPropertiesCommand(Command):
     def __init__(self, source, channel, delay=None, gain=None, eq_gain=None, eq_enable=None, dynamics_gain=None,
                  balance=None, volume=None, state=None):
         """
-        :param index: 0-indexed M/E number to control the preview bus of
+        :param source: The source index
+        :param channel: The channel index inside the source, -1 for the normal stereo channels and 0 and 1 for split
+        :param delay: Delay in frames
+        :param gain: Input gain
+        :param eq_gain: Make-up gain from the EQ dialog
+        :param eq_enable: Enable the EQ block
+        :param dynamics_gain: Make-up gain from the dynamics dialog
+        :param balance: Channel balance or pan
+        :param volume: The volume fader level
+        :param state: Bitfield for the mute and AFV state. 1=off, 2=on, 4=afv
         """
         self.source = source
         self.channel = channel
@@ -1344,7 +1363,7 @@ class FairlightStripPropertiesCommand(Command):
         split = 0xff if self.channel > -1 else 0x01
         self.channel = 0x00 if self.channel == -1 else self.channel
         pad = b'\xff\xff\xff\xff\xff\xff\xff'
-        data = struct.pack('>H H4x6sBb B 3x i ? 5x h 2x Hh 2x iB 3x', mask, self.source, pad, split, self.channel,
+        data = struct.pack('>H H4x6sBb B 3x i xx? 1x h 2x H 2x h 2x iB 3x', mask, self.source, pad, split, self.channel,
                            delay,
                            gain, eq_enable, eq_gain,
                            dynamics_gain, balance, volume, state)
@@ -1918,7 +1937,8 @@ class KeyPropertiesAdvancedChromaCommand(Command):
         green = 0 if self.green is None else self.green
         blue = 0 if self.blue is None else self.blue
 
-        data = struct.pack('>HBB HHH HH hhHhhh xx', mask, self.index, self.keyer, foreground, background, key_edge, spill,
+        data = struct.pack('>HBB HHH HH hhHhhh xx', mask, self.index, self.keyer, foreground, background, key_edge,
+                           spill,
                            flare, brightness, contrast, saturation, red, green, blue)
 
         return self._make_command('CACK', data)
@@ -2198,6 +2218,32 @@ class StreamingServiceSetCommand(Command):
         return self._make_command('CRSS', data)
 
 
+class StreamingAudioBitrateCommand(Command):
+    """
+    Implementation of the `STAB` command. This sets the audio bitrate parameters for the stream and recording.
+    In practice this is always set to 128k min/max bitrate.
+
+    ====== ==== ====== ===========
+    Offset Size Type   Description
+    ====== ==== ====== ===========
+    0      4    u32    Minimum bitrate (Bps)
+    4      4    u32    Maximum bitrate (Bps)
+    ====== ==== ====== ===========
+    """
+
+    def __init__(self, bitrate_min, bitrate_max):
+        """
+        :param bitrate_min: Minimum bitrate
+        :param bitrate_max: Maximum bitrate
+        """
+        self.bitrate_min = bitrate_min
+        self.bitrate_max = bitrate_max
+
+    def get_command(self):
+        data = struct.pack('>II', self.bitrate_min, self.bitrate_max)
+        return self._make_command('STAB', data)
+
+
 class StreamingStatusSetCommand(Command):
     """
     Implementation of the `StrR` command. This starts and stops the live stream
@@ -2757,6 +2803,22 @@ class TimeRequestCommand(Command):
         return self._make_command('TiRq', b'')
 
 
+class SetTimeOfDayCommand(Command):
+    """
+    Set the time of day on the hardware. Used my ATEMSC at startup to set the atem to the system clock of the computer.
+    """
+
+    def __init__(self, time):
+        if not isinstance(time, datetime.datetime):
+            raise ValueError()
+        self.time = time
+
+    def get_command(self):
+        timestamp = self.time.timestamp()
+        data = struct.pack('>I4x', timestamp)
+        return self._make_command('SToD', data)
+
+
 class TransferCompleteCommand(Command):
     """
     Implementation of the `*XFC` command. This is an command that's part of OpenSwitcher for the TCP protocol and not part
@@ -2786,3 +2848,101 @@ class TransferCompleteCommand(Command):
     def get_command(self):
         data = struct.pack('>HH ?xxx', self.store, self.slot, self.upload)
         return self._make_command('*XFC', data)
+
+
+class SupersourceBoxPropertiesCommand(Command):
+    """
+    Implementation of the `CSBP` command. This sets the state of the colorpicker for the advanced chroma keyer
+
+    ====== ==== ====== ===========
+    Offset Size Type   Description
+    ====== ==== ====== ===========
+    0      1    u8     Mask
+    1      1    u8     M/E index, 0-indexed
+    2      1    u8     Keyer index
+    3      1    bool   Enable cursor
+    4      1    bool   Enable preview
+    5      1    ?      padding
+    6      2    i16    Cursor X [-16000 - 16000]
+    8      2    i16    Cursor Y [-9000 - 9000]
+    10     2    u16    Cursor size [620 - 9925]
+    12     2    u16    Color Y
+    14     2    i16    Color Cb
+    16     2    i16    Color Cr
+    18     2    ?      padding
+    ====== ==== ====== ===========
+
+    === ==========
+    Bit Mask value
+    === ==========
+    0   Enabled
+    1   Source
+    2   X
+    3   Y
+    4   Size
+    5   Mask enable
+    6   Mask top
+    7   Mask bottom
+    8   Mask left
+    9   Mask right
+    === ==========
+
+    """
+
+    def __init__(self, index, box, enabled=None, source=None, x=None, y=None, size=None, masked=None, top=None,
+                 bottom=None,
+                 left=None, right=None):
+        """
+        :param index: 0-indexed M/E number to control the preview bus of
+        """
+        self.index = index
+        self.box = box
+        self.enabled = enabled
+        self.source = source
+        self.x = x
+        self.y = y
+        self.size = size
+        self.masked = masked
+        self.top = top
+        self.bottom = bottom
+        self.left = left
+        self.right = right
+
+    def get_command(self):
+        mask = 0
+        if self.enabled is not None:
+            mask |= 1 << 0
+        if self.source is not None:
+            mask |= 1 << 1
+        if self.x is not None:
+            mask |= 1 << 2
+        if self.y is not None:
+            mask |= 1 << 3
+        if self.size is not None:
+            mask |= 1 << 4
+        if self.masked is not None:
+            mask |= 1 << 5
+        if self.top is not None:
+            mask |= 1 << 6
+        if self.bottom is not None:
+            mask |= 1 << 7
+        if self.left is not None:
+            mask |= 1 << 8
+        if self.right is not None:
+            mask |= 1 << 9
+
+        enabled = False if self.enabled is None else self.enabled
+        source = 0 if self.source is None else self.source
+        x = 0 if self.x is None else self.x
+        y = 0 if self.y is None else self.y
+        size = 0 if self.size is None else self.size
+        masked = False if self.masked is None else self.masked
+        top = 0 if self.top is None else self.top
+        bottom = 0 if self.bottom is None else self.bottom
+        left = 0 if self.left is None else self.left
+        right = 0 if self.right is None else self.right
+
+        data = struct.pack('>HBB ?xH hh H?x HHHH', mask, self.index, self.box, enabled, source, x, y, size, masked, top,
+                           bottom, left, right)
+
+        return self._make_command('CSBP', data)
