@@ -60,18 +60,34 @@ class AtemProtocol:
         'AuxS': 'aux-output-source',
         'MPfe': 'mediaplayer-file-info',
         'MPCE': 'mediaplayer-selected',
+        'MPSp': 'mediaplayer-space',
+        'MPCS': 'mediaplayer-clip-source',
+        'MPAS': 'mediaplayer-audio-source',
+        'RCPS': 'mediaplayer-clip-status',
         'AMMO': 'audio-mixer-master-properties',
         'AMmO': 'audio-mixer-monitor-properties',
         'AMTl': 'audio-mixer-tally',
         'FASP': 'fairlight-strip-properties',
         'FAMP': 'fairlight-master-properties',
+        'FMPP': 'fairlight-properties',
         'FMHP': 'fairlight-headphones',
         'FAMS': 'fairlight-solo',
+        'AIXP': 'fairlight-expander-properties',
+        'AICP': 'fairlight-compressor-properties',
+        'AILP': 'fairlight-limiter-properties',
+        'MOCP': 'fairlight-master-compressor-properties',
+        'AMLP': 'fairlight-master-limiter-properties',
+        'ATMP': 'talkback-mixer-properties',
+        'TMIP': 'talkback-mixer-input-properties',
+        'MMOP': 'mix-minus-output-properties',
         '_TlC': 'tally-config',
         'TlIn': 'tally-index',
         'TlSr': 'tally-source',
         'FMTl': 'fairlight-tally',
         'MPrp': 'macro-properties',
+        'MRPr': 'macro-play-status',
+        'MRcS': 'macro-record-status',
+        'CCst': 'camera-control-settings',
         'AiVM': 'auto-input-video-mode',
         'FASD': 'fairlight-strip-delete',
         'FAIP': 'fairlight-audio-input',
@@ -87,6 +103,7 @@ class AtemProtocol:
         'SRSS': 'streaming-stats',
         'SAth': 'streaming-authentication',
         'AEBP': 'atem-eq-band-properties',
+        'AMBP': 'atem-master-eq-band-properties',
         'MvPr': 'multiviewer-properties',
         'MvIn': 'multiviewer-input',
         'VuMC': 'multiviewer-vu',
@@ -103,6 +120,16 @@ class AtemProtocol:
         'FDLv': 'fairlight-master-levels',
         'CCdP': 'camera-control-data-packet',
         '*XFC': 'transfer-complete',
+        '_SSC': 'supersource-config',
+        'SSrc': 'supersource-properties',
+        'SSBP': 'supersource-box-properties',
+        'V3sl': 'sdi-3g-level',
+        'RXMS': 'hyperdeck-settings',
+        'RXCP': 'hyperdeck-status',
+        'RXSS': 'hyperdeck-storage',
+        'RXCC': 'hyperdeck-clip-count',
+        'DCPV': 'displayclock-properties',
+        'DSTV': 'displayclock-set-time',
     }
 
     FIELDNAME_UNIQUE = {
@@ -143,6 +170,7 @@ class AtemProtocol:
         'multiviewer-vu': struct.Struct('>BB'),
         'multiviewer-safe-area': struct.Struct('>BB'),
         'camera-control-data-packet': struct.Struct('>BBB'),
+        'supersource-box-properties': struct.Struct('>B'),
     }
 
     def __init__(self, ip=None, port=9910, usb=None):
